@@ -1,8 +1,8 @@
 /*-------------------------------------------------------------------------*\
-* $Author: antanas $
-* $Date: 2019-11-15 20:06:25 +0200 (Fri, 15 Nov 2019) $ 
-* $Revision: 7424 $
-* $URL: svn://www.crystallography.net/cod-tools/tags/v2.7/src/components/codcif/cif_compiler.h $
+* $Author: andrius $
+* $Date: 2019-12-27 15:30:51 +0200 (Fri, 27 Dec 2019) $ 
+* $Revision: 7629 $
+* $URL: svn://www.crystallography.net/cod-tools/tags/v2.8/src/components/codcif/cif_compiler.h $
 \*-------------------------------------------------------------------------*/
 
 #ifndef __CIF_COMPILER_H
@@ -89,5 +89,6 @@ void add_tag_value( CIF_COMPILER *cif_cc, char *tag,
                     typed_value *tv, cexception_t *ex );
 
 CIF *new_cif_from_cif_file( char *filename, cif_option_t co, cexception_t *ex );
+CIF *new_cif_from_cif_string( char *buffer, cif_option_t co, cexception_t *ex );
 
 #endif
