@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
-**$Author: antanas $
-**$Date: 2019-11-15 20:06:25 +0200 (Fri, 15 Nov 2019) $ 
-**$Revision: 7424 $
-**$URL: svn://www.crystallography.net/cod-tools/tags/v2.7/src/components/codcif/cif_grammar_flex.h $
+**$Author: andrius $
+**$Date: 2019-12-27 14:57:31 +0200 (Fri, 27 Dec 2019) $ 
+**$Revision: 7625 $
+**$URL: svn://www.crystallography.net/cod-tools/tags/v2.8/src/components/codcif/cif_grammar_flex.h $
 \*---------------------------------------------------------------------------*/
 
 #ifndef __CIF_GRAMMAR_FLEX_H
@@ -17,8 +17,9 @@ typedef enum {
   CIF_FLEX_LEXER_FIX_NON_ASCII_SYMBOLS = 0x02,
   CIF_FLEX_LEXER_FIX_MISSING_CLOSING_DOUBLE_QUOTE = 0x04,
   CIF_FLEX_LEXER_FIX_MISSING_CLOSING_SINGLE_QUOTE = 0x08,
-  CIF_FLEX_LEXER_ALLOW_UQSTRING_BRACKETS = 0x16,
-  CIF_FLEX_LEXER_FIX_DATABLOCK_NAMES = 0x32,
+  CIF_FLEX_LEXER_ALLOW_UQSTRING_BRACKETS = 0x10,
+  CIF_FLEX_LEXER_FIX_DATABLOCK_NAMES = 0x20,
+  CIF_FLEX_LEXER_ALLOW_HIGH_CHARS = 0x40,
 } CIF_FLEX_LEXER_FLAGS;
 
 extern int yy_flex_debug;
@@ -38,6 +39,7 @@ void set_lexer_fix_missing_closing_double_quote( void );
 void set_lexer_fix_missing_closing_single_quote( void );
 void set_lexer_allow_uqstring_brackets( void );
 void set_lexer_fix_datablock_names( void );
+void set_lexer_allow_high_chars( void );
 
 void cif_flex_push_state( FILE *replace_yyin, cexception_t *ex );
 void cif_flex_pop_state( void );
