@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
 **$Author: andrius $
-**$Date: 2019-12-11 10:05:34 +0200 (Wed, 11 Dec 2019) $
-**$Revision: 7585 $
-**$URL: svn://www.crystallography.net/cod-tools/tags/v2.7/src/components/codcif/cif_lexer.c $
+**$Date: 2019-12-27 14:57:31 +0200 (Fri, 27 Dec 2019) $
+**$Revision: 7625 $
+**$URL: svn://www.crystallography.net/cod-tools/tags/v2.8/src/components/codcif/cif_lexer.c $
 \*---------------------------------------------------------------------------*/
 
 /* exports: */
@@ -592,13 +592,15 @@ static char *clean_string( char *src, int is_textfield, cexception_t *ex )
     cexception_t inner;
     cexception_guard( inner ) {
         while( *src != '\0' ) {
-            if( ( (*src & 255 ) < 32 || (*src & 255 ) >= 127 )
+            if( ( (*src & 255 ) < 32 || (*src & 255 ) == 127 ||
+                ( !cif_lexer_has_flags(CIF_FLEX_LEXER_ALLOW_HIGH_CHARS) &&
+                  (*src & 255 ) > 127 ) )
                 && (*src & 255 ) != '\n'
                 && (*src & 255 ) != '\t'
                 && (*src & 255 ) != '\r' ) {
                 if( cif_lexer_has_flags
                 (CIF_FLEX_LEXER_FIX_NON_ASCII_SYMBOLS)) {
-                    /* Do magic with non-ascii symbols */
+                    /* Do magic with non-ASCII symbols */
                     *dest = '\0';
                     length += DELTA;
                     new = reallocx( new, length + 1, &inner );
@@ -606,7 +608,7 @@ static char *clean_string( char *src, int is_textfield, cexception_t *ex )
                     dest = new + strlen( new ) - 1;
                     if( non_ascii_explained == 0 ) {
                         if( is_textfield == 0 ) {
-                            print_message( cif_cc, "WARNING", "non-ascii symbols "
+                            print_message( cif_cc, "WARNING", "non-ASCII symbols "
                                            "encountered in the text", ":",
                                            cif_flex_current_line_number(),
                                            cif_flex_current_position()+1,
@@ -614,7 +616,7 @@ static char *clean_string( char *src, int is_textfield, cexception_t *ex )
                             print_trace( cif_cc, (char*)cif_flex_current_line(),
                                          cif_flex_current_position()+1, ex );
                         } else {
-                            print_message( cif_cc, "WARNING", "non-ascii symbols "
+                            print_message( cif_cc, "WARNING", "non-ASCII symbols "
                                            "encountered in the text field -- "
                                            "replaced with XML entities", ":",
                                            cif_flex_current_line_number(),
@@ -627,8 +629,8 @@ static char *clean_string( char *src, int is_textfield, cexception_t *ex )
                     if( non_ascii_explained == 0 ) {
                         if( is_textfield == 0 ) {
                             ciferror( "incorrect CIF syntax" );
-                        } else if( non_ascii_explained == 0 ) {
-                            print_message( cif_cc, "ERROR", "non-ascii symbols "
+                        } else {
+                            print_message( cif_cc, "ERROR", "non-ASCII symbols "
                                            "encountered "
                                            "in the text field", ":",
                                            cif_flex_current_line_number(),
@@ -638,7 +640,7 @@ static char *clean_string( char *src, int is_textfield, cexception_t *ex )
                         }
                         non_ascii_explained = 1;
                     }
-                    dest--; /* Omit non-ascii symbols */
+                    dest--; /* Omit non-ASCII symbols */
                 }
             } else if( (*src & 255) == '\r' ) {
                 dest--; /* Skip carriage return symbols */
