@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
-** $Author: saulius $
-** $Date: 2015-07-21 17:39:27 +0300 (Tue, 21 Jul 2015) $
-** $Revision: 3587 $
-** $URL: svn://www.crystallography.net/cod-tools/tags/v2.7/src/externals/cexceptions/stdiox.c $
+** $Author: andrius $
+** $Date: 2019-12-27 15:30:51 +0200 (Fri, 27 Dec 2019) $
+** $Revision: 7629 $
+** $URL: svn://www.crystallography.net/cod-tools/tags/v2.8/src/externals/cexceptions/stdiox.c $
 \*---------------------------------------------------------------------------*/
 
 /* exports: */
@@ -37,3 +37,16 @@ void fclosex( FILE *file, cexception_t *ex )
               "could not close file", strerror( errno ));
     }
 }
+
+FILE *fmemopenx( void *buf, size_t size, const char *mode, cexception_t *ex )
+{
+    FILE *f = fmemopen( buf, size, mode );
+
+    if( f == NULL ) {
+        cexception_raise_syserror
+            ( ex, stdiox_subsystem, STDIOX_FILE_MEMOPEN_ERROR,
+              "could not open file in memory", strerror( errno ));
+    }
+
+    return f;
+}
