@@ -1,8 +1,8 @@
 /*---------------------------------------------------------------------------*\
-** $Author: saulius $
-** $Date: 2011-03-08 20:45:40 +0200 (Tue, 08 Mar 2011) $ 
-** $Revision: 1590 $
-** $URL: svn://www.crystallography.net/cod-tools/tags/v2.7/src/externals/cexceptions/stdiox.h $
+** $Author: andrius $
+** $Date: 2019-12-27 15:30:51 +0200 (Fri, 27 Dec 2019) $ 
+** $Revision: 7629 $
+** $URL: svn://www.crystallography.net/cod-tools/tags/v2.8/src/externals/cexceptions/stdiox.h $
 \*---------------------------------------------------------------------------*/
 
 #ifndef _STDIOX_H
@@ -17,11 +17,13 @@ typedef enum {
   STDIOX_OK = 0,
   STDIOX_FILE_OPEN_ERROR,
   STDIOX_FILE_CLOSE_ERROR,
+  STDIOX_FILE_MEMOPEN_ERROR,
 
   STDIOX_ERROR_last
 } STDIOX_ERROR;
 
 FILE *fopenx( const char *filename, const char *mode, cexception_t *ex );
 void fclosex( FILE *file, cexception_t *ex );
+FILE *fmemopenx( void *buf, size_t size, const char *mode, cexception_t *ex );
 
 #endif
