@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2019-12-08 23:12:08 +0200 (Sun, 08 Dec 2019) $
-#$Revision: 7569 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.7/src/lib/perl5/COD/CIF/Data/CIF2COD.pm $
+#$Date: 2019-12-18 16:19:20 +0200 (Wed, 18 Dec 2019) $
+#$Revision: 7618 $
+#$URL: svn://www.crystallography.net/cod-tools/tags/v2.8/src/lib/perl5/COD/CIF/Data/CIF2COD.pm $
 #------------------------------------------------------------------------------
 #*
 #  Parse a CIF file, prepare a COD database table entry from it.
@@ -23,7 +23,7 @@ use COD::CIF::Tags::Manage qw( cifversion get_data_value get_aliased_value );
 use COD::CIF::Tags::DictTags;
 use COD::Spacegroups::Names;
 use Scalar::Util qw( looks_like_number );
-use List::MoreUtils qw( uniq );
+use List::MoreUtils qw( uniq none any );
 
 require Exporter;
 our @ISA = qw( Exporter );
@@ -181,13 +181,17 @@ my %text_value_fields2tags = (
    'radiation'      => [ qw( _diffrn_radiation_probe ) ],
    'radType'        => [ qw( _diffrn_radiation_type ) ],
    'radSymbol'      => [ qw( _diffrn_radiation_xray_symbol ) ],
-   'duplicateof'    => [ qw( _cod_duplicate_entry
+   'duplicateof'    => [ qw( _cod_related_duplicate_entry.code
+                             _cod_related_duplicate_entry_code 
+                             _cod_duplicate_entry
                              _[local]_cod_duplicate_entry ) ],
-   'optimal'        => [ qw( _cod_related_optimal_struct
+   'optimal'        => [ qw( _cod_related_optimal_entry.code
+                             _cod_related_optimal_entry_code
+                             _cod_related_optimal_struct
                              _[local]_cod_related_optimal_struct ) ],
-   'status'         => [ qw( _cod_error_flag
-                             _[local]_cod_error_flag ) ],
-   'onhold'         => [ qw( _cod_hold_until_date ) ],
+   'onhold'         => [ qw( _cod_depositor.requested_release_date
+                             _cod_depositor_requested_release_date
+                             _cod_hold_until_date ) ],
 
    'title'          => [ qw( _publ_section_title ) ],
    'journal'        => [ qw( _journal_name_full ) ],
@@ -399,6 +403,7 @@ sub cif2cod
     # Fields that require a more complex logic
     $data{'file'}   = defined $cod_number ? $cod_number : $dataset->{'name'};
     $data{'flags'}  = get_cod_flags( $dataset );
+    $data{'status'} = get_cod_status( $dataset );
     $data{'method'} = get_experimental_method( $values );
 
     if (!defined $data{'vol'}) {
@@ -437,7 +442,6 @@ sub cif2cod
 
     $data{'Zprime'} = compute_z_prime( $data{'Z'}, $data{'sg'} );
 
-
     $data{'authors'} = get_authors( $dataset );
     $data{'text'}    = concat_text_field(\%data);
     $data{'acce_code'} =
@@ -556,6 +560,148 @@ sub get_cod_flags
     return join ',', @flags;
 }
 
+##
+# Determines the 'status' column value from the values of
+# the '_cod_entry_issue.severity' and '_cod_error_flag' data items.
+#
+# @param $data_block
+#       Reference to data block as returned by the COD::CIF::Parser.
+# @return
+#       Text string that should be inserted into the 'status' column or
+#       undef if such value could not be determined.
+##
+sub get_cod_status
+{
+    my ($data_block) = @_;
+
+    my %cod_status;
+
+    my $issue_status = get_cod_status_from_issue_severity($data_block);
+    if (defined $issue_status) {
+        $cod_status{$issue_status}++
+    }
+
+    my $flag_status = get_cod_status_from_error_flag($data_block);
+    if (defined $flag_status) {
+        $cod_status{$flag_status}++
+    }
+
+    return 'retracted' if exists $cod_status{'retracted'};
+    return 'errors'    if exists $cod_status{'errors'};
+    return 'warnings'  if exists $cod_status{'warnings'};
+
+    return;
+}
+
+##
+# Determines the 'status' column value from the value of
+# the '_cod_entry_issue.severity' data item.
+#
+# @param $data_block
+#       Reference to data block as returned by the COD::CIF::Parser.
+# @return
+#       Text string that should be inserted into the 'status' column or
+#       undef if such value could not be determined.
+##
+sub get_cod_status_from_issue_severity
+{
+    my ($data_block) = @_;
+
+    my $values = $data_block->{'values'};
+
+    my @issue_severity_tags = qw(
+        _cod_entry_issue.severity
+        _cod_entry_issue_severity
+    );
+
+    my $issue_severity_tag;
+    for my $tag ( @issue_severity_tags ) {
+        if ( exists $values->{$tag} ) {
+            $issue_severity_tag = $tag;
+            last;
+        }
+    }
+    return if !defined $issue_severity_tag;
+
+    my %severity_level;
+    for my $severity ( @{$values->{$issue_severity_tag}} ) {
+        $severity_level{$severity}++
+    };
+
+    my @known_severity_levels = qw(
+        note
+        warning
+        error
+        retraction
+    );
+
+    for my $severity (sort keys %severity_level) {
+        next if any {$_ eq $severity} @known_severity_levels;
+        warn "data item '$issue_severity_tag' value '$severity' does " .
+             'not belong to the set of known severity levels [' .
+             ( join ', ', map { "'$_'" } @known_severity_levels ) . ']' .
+             ' -- value will be ignored' . "\n";
+    }
+
+    return 'retracted' if exists $severity_level{'retraction'};
+    return 'errors'    if exists $severity_level{'error'};
+    return 'warnings'  if exists $severity_level{'warning'};
+
+    return;
+}
+
+##
+# Determines the 'status' column value from the value of
+# the '_cod_error_flag' data item.
+#
+# @param $data_block
+#       Reference to data block as returned by the COD::CIF::Parser.
+# @return
+#       Text string that should be inserted into the 'status' column or
+#       undef if such value could not be determined.
+##
+sub get_cod_status_from_error_flag
+{
+    my ($data_block) = @_;
+
+    my $values = $data_block->{'values'};
+
+    my @error_flag_tags = qw(
+        _cod_error_flag
+        _[local]_cod_error_flag
+    );
+
+    my $error_flag_tag;
+    for my $tag ( @error_flag_tags ) {
+        if ( exists $values->{$tag} ) {
+            $error_flag_tag = $tag;
+            last;
+        }
+    }
+    return if !defined $error_flag_tag;
+
+    my $status = $values->{$error_flag_tag}[0];
+    my @known_error_flags = qw(
+        none
+        warnings
+        errors
+        retracted
+    );
+
+    if ( none { $_ eq $status } @known_error_flags ) {
+        warn "data item '$error_flag_tag' value '$status' does " .
+             'not belong to the set of known error flags [' .
+             ( join ' ,', map {"'$_'"} @known_error_flags ) . '] ' .
+             '-- value will be ignored' . "\n";
+    }
+
+    return 'retracted' if $status eq 'retracted';
+    return 'errors'    if $status eq 'errors';
+    return 'warnings'  if $status eq 'warnings';
+
+    return;
+}
+
 sub get_authors
 {
     my ( $dataset ) = @_;
@@ -701,9 +847,13 @@ sub get_experimental_method
 {
     my ($values) = @_;
 
-    if( exists $values->{'_cod_struct_determination_method'} ) {
-        return $values->{'_cod_struct_determination_method'}[0];
-    }
+    my @determination_method_tags = qw(
+        _cod_structure.determination_method
+        _cod_structure_determination_method
+        _cod_struct_determination_method
+    );
+    my $method = get_aliased_value($values, \@determination_method_tags);
+    return $method if defined $method;
 
     my @powder_tags = grep { /^_pd_/ }
                         @COD::CIF::Tags::DictTags::tag_list;
@@ -743,7 +893,8 @@ sub get_coeditor_code
     # the coeditor code from the orignal file name
     if ( !defined $acce_code && defined $journal_name &&
          $journal_name =~ /^Acta Cryst/ ) {
-        for ( qw( _cod_data_source_file
+        for ( qw( _cod_data_source.file
+                  _cod_data_source_file
                   _[local]_cod_data_source_file ) ) {
             if( exists $values->{$_} ) {
                 $acce_code = get_data_value( $values, $_, 0 );
