@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2019-11-15 17:58:23 +0200 (Fri, 15 Nov 2019) $
-#$Revision: 7411 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.7/src/lib/perl5/COD/CIF/Data/CODNumbers.pm $
+#$Date: 2020-01-01 08:42:18 +0200 (Wed, 01 Jan 2020) $
+#$Revision: 7637 $
+#$URL: svn://www.crystallography.net/cod-tools/tags/v2.8/src/lib/perl5/COD/CIF/Data/CODNumbers.pm $
 #------------------------------------------------------------------------------
 #*
 #  Find COD numbers of duplicate structures.
@@ -14,7 +14,7 @@ use strict;
 use warnings;
 use DBI;
 use File::Basename qw( basename );
-use List::MoreUtils qw( uniq );
+use List::MoreUtils qw( any uniq );
 use COD::Formulae::Parser::AdHoc;
 use COD::DateTime qw( parse_datetime );
 use COD::CIF::Data::CellContents qw( cif_cell_contents );
@@ -360,8 +360,14 @@ sub cif_fill_data
                 _pd_prep_temperature
             )
         ],
+        # FIXME: the 'enantiomer' field should be removed prior to a major
+        # release since it was replaced with the 'related_enantiomer_entries'
+        # field. The new field takes into consideration that there might be
+        # more than one related enantiomer entry.
         'enantiomer' => [
             qw(
+                _cod_related_enantiomer_entry.code
+                _cod_related_enantiomer_entry_code
                 _cod_enantiomer_of
             )
         ],
@@ -383,13 +389,25 @@ sub cif_fill_data
         }
     }
 
+    my $enantiomer_entry_codes = get_related_enantiomer_entry_codes( $dataset );
+    if (@{$enantiomer_entry_codes}) {
+        $structure{'related_enantiomer_entries'} = $enantiomer_entry_codes;
+    }
+
     my $value = get_aliased_value($values,
-                    [ qw( _cod_related_optimal_struct
+                    [ qw( _cod_related_optimal_entry.code
+                          _cod_related_optimal_entry_code
+                          _cod_related_optimal_struct
                           _[local]_cod_related_optimal_struct ) ]);
     if (defined $value) {
         $structure{'related_optimal'} = $value;
     }
 
+    # FIXME: whether a structure is suboptimal can be judged not only from
+    # the value of the _cod_suboptimal_structure data item, but also from
+    # the presence of related optimal entry data items (the preferred way).
+    # This type of functionality is supplied by the
+    # COD::CIF::Data::CODFlags::is_suboptimal() subroutine
     $value = get_aliased_value($values,
                 [ qw( _cod_suboptimal_structure
                       _[local]_cod_suboptimal_structure ) ] );
@@ -400,6 +418,35 @@ sub cif_fill_data
     return \%structure;
 }
 
+##
+# Returns the COD IDs of entries that are explicitly referenced as
+# enantiomers in the provided data block.
+#
+# @param $data_block
+#       Reference to data block as returned by the COD::CIF::Parser.
+# @return \@related_enantiomer_entry_codes
+#       Reference to an array of COD IDs.
+##
+sub get_related_enantiomer_entry_codes
+{
+    my ($data_block) = @_;
+
+    my @related_enantiomer_tags = qw(
+        _cod_related_enantiomer_entry.code
+        _cod_related_enantiomer_entry_code
+        _cod_enantiomer_of
+    );
+
+    my @related_enantiomer_entry_codes;
+    for my $tag (@related_enantiomer_tags) {
+        next if !defined $data_block->{'values'}{$tag};
+        push @related_enantiomer_entry_codes, @{$data_block->{'values'}{$tag}};
+    }
+    @related_enantiomer_entry_codes = uniq @related_enantiomer_entry_codes;
+
+    return \@related_enantiomer_entry_codes;
+}
+
 #------------------------------------------------------------------------------
 
 sub get_cell($)
@@ -809,28 +856,20 @@ sub entries_are_the_same
         }
     };
 
+    return 0 if entries_are_enantiomers( $entry1, $entry2 );
+
     # FIXME: the logic involving optimal/suboptimal structures seems
     # to be broken. For example, structures that contain the
     # _cod_suboptimal_structure data are not even recognised as themselves
     # by the cif_cod_numbers script
-    my $are_the_same =
-        have_equiv_lattices( $entry1, $entry2, \%options ) &&
-        have_equiv_conditions( $entry1, $entry2 ) &&
-        (!defined $entry1->{suboptimal} || $entry1->{suboptimal} ne 'yes') &&
-        (!defined $entry2->{suboptimal} || $entry2->{suboptimal} ne 'yes');
+    return 0 if defined $entry1->{'suboptimal'} && $entry1->{'suboptimal'} eq 'yes';
+    return 0 if defined $entry2->{'suboptimal'} && $entry2->{'suboptimal'} eq 'yes';
 
-    # This is a temporary statement that should be rewritten once the
-    # issues involving the suboptimal structures are resolved
-    return 0 if (!$are_the_same);
+    return 0 if !have_equiv_lattices( $entry1, $entry2, \%options );
+    return 0 if !have_equiv_conditions( $entry1, $entry2 );
 
-    # FIXME: the enantiomer and related optimal checks are parameter position
-    # dependent:
+    # FIXME: the related optimal check is parameter position dependent:
     # ( entries_are_the_same($s1, $s2) != entries_are_the_same($s1, $s2) )
-    if ( defined $entry1->{'enantiomer'}{'_cod_enantiomer_of'} &&
-         $entry1->{'enantiomer'}{'_cod_enantiomer_of'} eq $entry2->{'id'} ) {
-        return 0;
-    }
-
     if ( defined $entry1->{'related_optimal'} &&
          $entry1->{'related_optimal'} eq $entry2->{'id'} &&
          defined $entry1->{'suboptimal'} &&
@@ -856,6 +895,74 @@ sub entries_are_the_same
     return 1;
 }
 
+##
+# Evaluates if at least one of the entries is explicitly marked by the COD
+# maintainers as an enantiomer of the other. 
+#
+# @param $entry_1
+#       Data structure of the first entry as returned by the 'cif_fill_data()'
+#       or the 'get_database_entries()' subroutines.
+# @param $entry_2
+#       Data structure of the second entry as returned by the 'cif_fill_data()'
+#       or the 'get_database_entries()' subroutines.
+# @return
+#       '1' if at least one of the entries is marked as an enantiomer of the other,
+#       '0' otherwise.
+##
+sub entries_are_enantiomers
+{
+    my ($entry_1, $entry_2) = @_;
+
+    return 1 if is_related_enantiomer_entry($entry_1, $entry_2);
+    return 1 if is_related_enantiomer_entry($entry_2, $entry_1);
+
+    return 0;
+}
+
+##
+# Evaluates if the main entry references the related entry as an enantiomer.
+#
+# @param $main_entry
+#       Data structure of the main entry as returned by the 'cif_fill_data()'
+#       or the 'get_database_entries()' subroutines.
+# @param $related_entry
+#       Data structure of the relared entry as returned by the 'cif_fill_data()'
+#       or the 'get_database_entries()' subroutines.
+# @return
+#       '1' if the main entry references the related entry as an enantiomer,
+#       '0' otherwise.
+##
+sub is_related_enantiomer_entry
+{
+    my ($main_entry, $related_entry) = @_;
+
+    my @enantiomer_entries;
+
+    # FIXME: the following code block should be removed once
+    # the 'enantiomer' entry field is properly deprecated and removed 
+    # BEGIN
+    if (defined $main_entry->{'enantiomer'}) {
+        my @enantiomer_tags = qw(
+            _cod_related_enantiomer_entry.code
+            _cod_related_enantiomer_entry_code
+            _cod_enantiomer_of
+        );
+        for my $tag (@enantiomer_tags) {
+            next if !defined $main_entry->{'enantiomer'}{$tag};
+            push @enantiomer_entries, $main_entry->{'enantiomer'}{$tag};
+        }
+    }
+    # END
+
+    if (exists $main_entry->{'related_enantiomer_entries'}) {
+        push @enantiomer_entries, @{$main_entry->{'related_enantiomer_entries'}};
+    }
+
+    my $is_enantiomer = any { $_ eq $related_entry->{'id'} } @enantiomer_entries;
+
+    return $is_enantiomer ? 1 : 0;
+}
+
 ##
 # Connects to a database using the provided parameters. This subroutine
 # acts as a wrapper for the 'DBI->connect' subroutine.
